﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.ComponentModel;
using System.Windows.Input;
using Infragistics.Windows;
using Infragistics.Windows.Helpers;
using Infragistics.Windows.Editors;
using System.Windows.Controls;

namespace Infragistics.Windows.Samples
{
    /// <summary>
    /// Custom <see cref="ValueEditor"/> used to host arbitrary elements.
    /// </summary>
    [TemplatePart(Name = "PART_FocusBefore", Type = typeof(FrameworkElement))]
    [TemplatePart(Name = "PART_FocusAfter", Type = typeof(FrameworkElement))]
    public class ControlHostEditor : ValueEditor
    {
        #region Member Variables

        private FrameworkElement _focusBefore;
        private FrameworkElement _focusAfter;
        private IInputElement _focusSite;

        #endregion //Member Variables

        #region Constructor
        static ControlHostEditor()
        {
            FrameworkElement.DefaultStyleKeyProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(typeof(ControlHostEditor)));
            Control.HorizontalContentAlignmentProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.HorizontalAlignmentStretchBox));
            Control.VerticalContentAlignmentProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(KnownBoxes.VerticalAlignmentStretchBox));
            ValueEditor.ValueTypeProperty.OverrideMetadata(typeof(ControlHostEditor), new FrameworkPropertyMetadata(typeof(object)));
        }

        /// <summary>
        /// Initializes a new <see cref="ControlHostEditor"/>
        /// </summary>
        public ControlHostEditor()
        {
        }
        #endregion //Constructor

        #region Base class overrides

        #region Can(Render|Edit)Type
        /// <summary>
        /// Indicates if the editor can handle the specified type
        /// </summary>
        /// <param name="type">The type to evaluate</param>
        /// <returns>Always returns true since it is up to the host template to actually edit the value.</returns>
        public override bool CanEditType(Type type)
        {
            return true;
        }

        /// <summary>
        /// Indicates if the editor can handle the specified type
        /// </summary>
        /// <param name="type">The type to evaluate</param>
        /// <returns>Always returns true since it is up to the host template to actually render the value.</returns>
        public override bool CanRenderType(Type type)
        {
            return true;
        }

        #endregion //Can(Render|Edit)Type

        #region OnApplyTemplate
        /// <summary>
        /// Invoked when the template has been applied to the element.
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _focusBefore = this.GetTemplateChild("PART_FocusBefore") as FrameworkElement;
            _focusAfter = this.GetTemplateChild("PART_FocusAfter") as FrameworkElement;

            if (this.IsEmbedded == false)
            {
                if (null != _focusBefore)
                    _focusBefore.Visibility = Visibility.Collapsed;

                if (null != _focusAfter)
                    _focusAfter.Visibility = Visibility.Collapsed;
            }
        }

        #endregion //OnApplyTemplate

        #region OnFocusSiteChanged
        /// <summary>
        /// Invoked when the FocusSite has changed for the editor.
        /// </summary>
        protected override void OnFocusSiteChanged()
        {
            base.OnFocusSiteChanged();

            if (null != _focusSite)
            {
                _focusSite.GotKeyboardFocus -= new KeyboardFocusChangedEventHandler(OnFocusSiteGotKeyboardFocus);
            }

            _focusSite = this.FocusSite as IInputElement;

            if (null != _focusSite)
            {
                _focusSite.GotKeyboardFocus += new KeyboardFocusChangedEventHandler(OnFocusSiteGotKeyboardFocus);
            }
        }
        #endregion //OnFocusSiteChanged

        #region OnKeyDown
        /// <summary>
        /// Invoked when a key is pressed within the editor.
        /// </summary>
        /// <param name="e">Provides information about the event</param>
        protected override void OnKeyDown(System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.Tab && this.IsEmbedded)
            {
                bool next;

                if (Keyboard.Modifiers == ModifierKeys.None)
                    next = true;
                else if (Keyboard.Modifiers == ModifierKeys.Shift)
                    next = false;
                else
                    return;

                // this editor wants to handle the tab key to shift focus
                // to the next/previous element. if focus is shifted to
                // an element within the edit template then we want to mark 
                // the key as processed. if focus is shift to our element 
                // before/after the edit template elements then we want to 
                // allow the tab key to bubble up so the datagrid can process
                // it
                IInputElement originalFocus = Keyboard.FocusedElement;

                if (ControlHostEditor.MoveFocus(originalFocus, next))
                {
                    if (originalFocus != Keyboard.FocusedElement &&
                        Keyboard.FocusedElement != _focusBefore &&
                        Keyboard.FocusedElement != _focusAfter)
                    {
                        e.Handled = true;
                    }
                }
            }

            base.OnKeyDown(e);
        }

        #endregion //OnKeyDown

        #endregion //Base class overrides

        #region Properties

        #endregion //Properties

        #region Methods

        #region MoveFocus
        private static bool MoveFocus(IInputElement src, bool next)
        {
            return MoveFocus(src, next ? FocusNavigationDirection.Next : FocusNavigationDirection.Previous);
        }

        private static bool MoveFocus(IInputElement src, FocusNavigationDirection direction)
        {
            FrameworkElement fe = src as FrameworkElement;
            TraversalRequest req = new TraversalRequest(direction);

            if (null != fe)
                return fe.MoveFocus(req);

            FrameworkContentElement fce = src as FrameworkContentElement;

            if (null != fce)
                return fce.MoveFocus(req);

            return false;
        }
        #endregion //MoveFocus

        #region OnFocusSiteGotKeyboardFocus
        private void OnFocusSiteGotKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
        {
            if (e.NewFocus == _focusSite)
            {
                // this isn't ideal but if someone shift tab's into the edit then focus
                // the last child element
                FocusNavigationDirection dir = Keyboard.IsKeyDown(Key.Tab) && Keyboard.Modifiers == ModifierKeys.Shift
                    ? FocusNavigationDirection.Last
                    : FocusNavigationDirection.First;

                MoveFocus(_focusSite, dir);
            }
        }
        #endregion //OnFocusSiteGotKeyboardFocus

        #endregion //Methods
    }
}
